;ComboJos.asm - Design patterned after combolok.asm

;For use with 1.25 to 1.5 volt AA battery operation.  Unit goes to 
;   sleep when not in use.

;  New BUTTON for ButtonPush 
;  
;4 bytes (32 bits) reserved for opening code.  One or all can be used.
;EEPROM contains permanent code which must be programmed before use
;  (first byte contains number of bits programmed)

;Dit-Dah entries used for code to open lock.

;HoldDown count of 5 goes into programming mode to set up new entry code.
;   Dit-Dah entries from one up to 255 are permitted.

;Current draw at idle = approx 1.6 ma (@5.5 volts)

;Internal oscillator in ATTINY2313 is running micro-controller at 1MHz

;January 30, 2006

;----------------------------------------------------

;       HOLDDOWN, LED FLASH, COMMANDS:

;5. Enter new code using dit-dah entries. One to 255 dit-dah entries possible.
;    (this is entered into EEPROM for permanent storage after a power down)

;r0-r4 = SRAM TP code entry registers (r0=No. of bits, r1-4 = 4 bytes for code)
;r5-r9 = SRAM duplicate code copied from EEPROM (r5= No. of bits, r6-9 = 4 bytes)
;r10-r12 = Long Delay countdown "escape" timer registers

;-------------------------------------------------------


.include "tn2313def.inc"

rjmp	START1
rjmp	Decode


START1:				;set up stack pointer and initialize cold start

ldi		r16,low(RAMEND)		; Load low byte address of end of RAM into register R16
out		SPL,R16				; Initialize stack pointer to end of internal RAM

ldi		r16,7		;Set bits 0-3 for output
out		DDRA,r16	;Execute data direction with PA0-3 as output

clr		r16
out		PortA,r16	;turn off ports

ldi		R16,251		;Set bit 2(4) as input ... remaining bits as output
out		DDRD,R16	;execute data direction with bit 2 as input
out		PORTD,R16	;execute to set out ports high, and in ports with
					; no input resistor

in		r16,GIMSK	;enable INT0 (D port, bit 2(4))
ori		r16,64		; (bit 6 = INT0)
out		GIMSK,r16

in		r16,MCUCR	;set up MCUCR for power down and low level sensing
ori		r16,16		;  power down bit set to one = power down
andi	r16,252		;  low level sensing for INT0 (bits 0,1 to zero)
out		MCUCR,r16

ser		R16			;Set all bits for data direction as outputs
out		DDRB,R16	;execute data direction with all as outputs
clr		r16
out		PORTB,R16	;execute to set out ports low

clr		r18			;clear flags register R18

rcall	FetchEEPROM	;grab last recorded code data and put in R5-r9

bset	7			;enable global interrupts


;-----------------------------

PowerDown:		;Power down and loop to await interrupt

in		r16,MCUCR	;Set SE bit 5(32) for entering sleep mode
ori		r16,32
out		MCUCR,r16
sleep				;Go to sleep - power down mode

rjmp	PowerDown


;--------------------------------------

Decode:		;Interrupt Routine: Decodes touches from touch plate


push	r16			;save value in r16

in		r16,SREG
sts		$60,r16		;Save SREG contents in RAM

in		r16,MCUCR	;Clear SE bit 5(32) to prevent accidental sleep
andi	r16,223		
out		MCUCR,r16

rcall	Button

cpi		r20,5		;compare to see if holddown count is FIVE
breq	Decode2

rcall	CheckMatch
sbrs	r18,0		;skip if bit 0(1) (a match) in flags register is set
rjmp	Decode3

clr		r10
clr		r11
clr		r12			;clear all long delay count registers
ldi		r16,4		;maximum number of iterations - long count (5.7 MIN)
sbi		PortB,0		;turn on LED


Decode4:

sbis	PIND,2			;skip if switch is up, d2(4)
rjmp	Decode5

dec		r10
brne	Decode4

dec		r11
brne	Decode4

dec		r12
brne	Decode4

dec		r16
brne	Decode4


Decode6:

cbi		PortB,0			;off LED


DecodeEnd:

lds		r16,$60			;retrieve SREG
out		SREG,r16
pop		r16				;restore r16 value

reti					;return from interrupt


Decode2:

rcall	FlashFour


Decode2a:

sbic	PIND,2
rjmp	Decode2a

rcall	Button
rcall	WriteEEPROM
mov		r5,r0
mov		r6,r1
mov		r7,r2
mov		r8,r3
mov		r9,r4		;move data code into registers
rcall	FlashFour	;flash LED four times on exit

rcall	DecodeEnd


Decode3:

rcall	FlashFour		;rapid 4 flashes of LED
rjmp	DecodeEnd


Decode5:

sbic	PIND,2			;skip if switch is down
rjmp	Decode6

rcall	Delay40th
rjmp	Decode5
;------------------------

Delay40th:			; 1/40th second delay

push	r31
push	r0
push	r1

ldi		r31,33		; high byte
mov		r1,r31		; set up r1 for 1/40th second delay
clr		r0
ldi		r31,1		;no delay for r31


DelayA:

dec		r0
brne	DelayA

dec		r1
brne	DelayA

dec		r31
brne	DelayA

pop		r1
pop		r0
pop		r31

ret

;----------------------------------

DelayThreeFourths:	;3/4 second delay

push	r16
ldi		r16,30
rjmp	Delay6a


;----------------------------------


DelayOne:		;1 second delay

push	r16
ldi		r16,40
rjmp	Delay6a

;------------------------------------


Delay6:				;6 second delay

push	r16
clr		r16


Delay6a:

rcall	Delay40th
dec		r16
brne	Delay6a

pop		r16
ret

;---------------------------------------


Delay15:			;15 second delay routine

push 	r0
push	r16
clr		r0
ldi		r16,3
mov		r0,r16
clr		r16


Delay15a:

rcall	Delay40th	;1/40th second delay
dec		r16
brne	Delay15a

dec		r0
brne	Delay15a

pop		r16
pop		r0
ret


;------------------------------------

FlashOnce:

push	r16
ldi		r16,1
rjmp	FF1


;---------------------------

FlashFour:			;rapid four flashes of LED

push r16

ldi	r16,4


FF1:

rcall	Delay40th
sbi		PortB,0		;on LED
rcall	Delay40th
cbi		PortB,0		;off LED
rcall	Delay40th

dec		r16
brne	FF1

pop		r16
ret


;-----------------------------------

VariableFlashBoth:

VF1:

sbi		PortB,0		;on PROGRAM LED
rcall	Delay40th
cbi		PortB,0		;off PROGRAM LED
rcall	Delay40th

dec		r16
brne	VF1

ret


;---------------------------------------

Button:		;Replaces old ButtonPush.  1/11/2006

push	r23

clr		r20		;HoldDown count
clr		r0		;clear number of bits in code
clr		r1		;clear lowest code count
clr		r2		;clear next code count
clr		r3		;clear next code count
clr		r4		;clear next code count (clear all code counts)

clr		r23


B1:

inc		r23
rcall	Delay40th
cpi		r23,120		;delay before first holddown count - 40=one second - 3 seconds
brsh	B3			;branch if - it is a holddown count

sbis	PIND,2		;skip if switch is NOT down
rjmp	B1


B2:

cpi		r23,20		;compare r23 for 20 (1/2 second dah count)
brlo	B2b			;branch if lower (dit count)


B2a:				;it is dah count

sec					;set carry flag -- it is dah entry
rol		r1
rol		r2			;rotate data into registers r1-r4
rol		r3
rol		r4

sbi		PortB,0
rcall	Delay40th
rcall	Delay40th	;delay for longer on of LEDs
rcall	Delay40th
rcall	Delay40th


B2c:

rcall	Delay40th
cbi		PortB,0

inc		r0			;increment bit count
clr		r23			;clear r23


B2d:

sbis	PIND,2		;skip if switch is up
rjmp	B2e			;jump back - there is a new entry coming in

inc		r23
rcall	Delay40th
cpi		r23,120		;has switch been up 120 counts? 40=one second (3 seconds)
brsh	B5

rjmp	B2d


B2e:

clr		r23
rjmp	B1


B2b:				;Dit count detected

clc					;clear carry flag -- it is dit entry
rol		r1
rol		r2			;rotate data into registers r1-r4
rol		r3
rol		r4

sbi		PortB,0

rjmp	B2c


B3:					;Holddown count has been detected

inc		r20			;increment holddown count
sbi		PortB,0
rcall	Delay40th
cbi		PortB,0		;flash PROGRAM LED

clr		r23			;reset r23 to zero count


B4:

inc		r23
rcall	Delay40th
sbic	PIND,2		;skip if switch is down
rjmp	B4a

cpi		r23,30
brlo	B4

rjmp	B3


B4a:

rcall	Delay40th


B5:

pop		r23
ret


;---------------------------------------

CheckMatch:			;Check for match between r0-4 and r5-r9

cp		r0,r5		;compare number of digits present
brne	CMOUT1

cp		r1,r6
brne	CMOUT1

cp		r2,r7
brne	CMOUT1

cp		r3,r8
brne	CMOUT1

cp		r4,r9
brne	CMOUT1

sbr		r18,1		;set r18,0(1) - there is a match
ret	


CMOUT1:

cbr		r18,1		;clear r18,0(1) - no match
ret


;-----------------------------------

FetchEEPROM:		;grabs data in permanent saved EEPROM

ldi		r17,0		;start of EEPROM memory location

FEE:

sbic	EECR,EEWE	;Wait for completion of previous write
rjmp	FEE

out		EEAR,r17	;set up address in address register
sbi		EECR,EERE	;start EEPROM read by writing EERE

in		r5,EEDR		;read data from data register

inc		r17			;increment to next EEPROM address

out		EEAR,r17	;set up address in address register
sbi		EECR,EERE	;start EEPROM read by writing EERE

in		r6,EEDR		;read data from data register

inc		r17			;increment to next EEPROM address

out		EEAR,r17	;set up address in address register
sbi		EECR,EERE	;start EEPROM read by writing EERE

in		r7,EEDR		;read data from data register

inc		r17			;increment to next EEPROM address

out		EEAR,r17	;set up address in address register
sbi		EECR,EERE	;start EEPROM read by writing EERE

in		r8,EEDR		;read data from data register

inc		r17			;increment to next EEPROM address

out		EEAR,r17	;set up address in address register
sbi		EECR,EERE	;start EEPROM read by writing EERE

in		r9,EEDR		;read data from data register

ret					;code data now in r5-9


;-------------------------------------------------

WriteEEPROM:		;write new code from r0-r4 into EEPROM

ldi		r17,0		;start of EEPROM memory section

WEE:

sbic	EECR,EEPE
rjmp	WEE

out		EEAR,r17	;set up address in address register
out		EEDR,r0		;put contents of r0 into EEPROM data location

sbi		EECR,EEMPE	;write logical one to EEMPE

sbi		EECR,EEPE	;start eeprom write by setting EEPE

inc		r17


WEE1:

sbic	EECR,EEPE
rjmp	WEE1

out		EEAR,r17	;set up address in address register
out		EEDR,r1		;put contents of r1 into EEPROM data location

sbi		EECR,EEMPE	;write logical one to EEMPE

sbi		EECR,EEPE	;start eeprom write by setting EEPE

inc		r17


WEE2:

sbic	EECR,EEPE
rjmp	WEE2

out		EEAR,r17	;set up address in address register
out		EEDR,r2		;put contents of r2 into EEPROM data location

sbi		EECR,EEMPE	;write logical one to EEMPE

sbi		EECR,EEPE	;start eeprom write by setting EEPE

inc		r17


WEE3:

sbic	EECR,EEPE
rjmp	WEE3

out		EEAR,r17	;set up address in address register
out		EEDR,r3		;put contents of r3 into EEPROM data location

sbi		EECR,EEMPE	;write logical one to EEMPE

sbi		EECR,EEPE	;start eeprom write by setting EEPE

inc		r17


WEE4:

sbic	EECR,EEPE
rjmp	WEE4

out		EEAR,r17	;set up address in address register
out		EEDR,r4		;put contents of r4 into EEPROM data location

sbi		EECR,EEMPE	;write logical one to EEMPE

sbi		EECR,EEPE	;start eeprom write by setting EEPE

ret


;--------------------------------------------------

TEST1:

rcall	DelayOne
rcall	DelayOne
tst		r26
breq	T1b

T1a:

sbi		PortB,3
rcall	Delay40th
cbi		PortB,3

rcall	DelayThreeFourths
dec		r26
brne	T1a

rcall	DelayOne
ret


T1b:

sbi		PortB,3
rcall	Delay6
cbi		PortB,3
ret


;----------------------------------------